function gather_feats(save_filepath, varargin)
% Please see README.md in this directory. Gathers features and labels for 
% running LAD boosting.
%
% @authors:     Fuxin Li, Ahmad Humayun
% @contact:     ahumayun@cc.gatech.edu
% @affiliation: Georgia Institute of Technology
% @date:        Fall 2013 - Spring 2015

    % before running it make sure filter_points_major_dir.cpp is compiled
    assert(~isempty(which('filter_points_major_dir')), ...
           'run ''mex -O filter_points_major_dir.cpp''');
    
    % set default save filepath
    if ~exist('save_filepath', 'var'), save_filepath = 'trainval.mat'; end
    
    % Download BSD 500:
    % http://www.eecs.berkeley.edu/Research/Projects/CS/vision/grouping/BSR/BSR_full.tgz
    bsr_path = '/data/images/arbelaez_PAMI_2011_gpb';
    fuxin_lib_dir = '/home/ahumayun/videovolumes/fuxin_lib_src';

    % add codes to path
    rigor_root_dir = fullfile(fileparts(which(mfilename)), '..');
    addpath(rigor_root_dir);
    addpath(fullfile(rigor_root_dir, 'utils'));
    addpath(genpath(fuxin_lib_dir));
    addpath(fullfile(bsr_path, 'bench/benchmarks'));
    
    % gather features for train and validation sets
    [train_feats, train_out, train_extra_feats] = ...
                    gather_feats_dataset(bsr_path, 'train', varargin{:});
    [val_feats, val_out, val_extra_feats] = ...
                    gather_feats_dataset(bsr_path, 'val', varargin{:});

    % collare and store data
    train_feats = cell2mat(train_feats')';
    val_feats = cell2mat(val_feats')';
    train_extra_feats = cell2mat(train_extra_feats);
    val_extra_feats = cell2mat(val_extra_feats);
    train_out = cell2mat(train_out);
    val_out = cell2mat(val_out);
    
    save(save_filepath);
end
