function [edgelet_sp, output] = gen_sp_extra_feats(sp_seg, I)
% Please see README.md in this directory. This function generates features 
% based on pixel-wise edge strengths First get to just percentile features.

% get the indexes for the pairs of pixels to compute pairwise
%  potentials over
[bndry_pairs, edgelet_ids, edgelet_sp] = ...
    generate_sp_neighborhood(sp_seg);
% Accumulate brightness histogram and LAB histogram separately
labI = rgb2lab(I);
bright_hist = accumulate_brightness_on_sp(I,sp_seg, 20);
l_hist = accumulate_brightness_on_sp(labI(:,:,1), sp_seg, 20);
a_hist = accumulate_brightness_on_sp(labI(:,:,2), sp_seg, 20);
b_hist = accumulate_brightness_on_sp(labI(:,:,3), sp_seg, 20);

output(:,1) = norm_chi2_dist(bright_hist, edgelet_sp);
output(:,2) = norm_chi2_dist(l_hist, edgelet_sp);
output(:,3) = norm_chi2_dist(a_hist, edgelet_sp);
output(:,4) = norm_chi2_dist(b_hist, edgelet_sp);

end
        
function [bndry_pairs, edgelet_ids, edgelet_sp] = ...
    generate_sp_neighborhood(sp_seg)
% create offset images to find superpixel boundary pixels
south_seg = [sp_seg(2:end,:); sp_seg(end,:)];
east_seg = [sp_seg(:,2:end), sp_seg(:,end)];

% find the actual boundary pixels
south_bndry = south_seg ~= sp_seg;
east_bndry = east_seg ~= sp_seg;

% find the src and the dst superpixel for each boundary pixel
south_sp_src = sp_seg(south_bndry);
south_sp_dst = south_seg(south_bndry);
east_sp_src = sp_seg(east_bndry);
east_sp_dst = east_seg(east_bndry);

south_bndry_pairs = find(south_bndry);
east_bndry_pairs = find(east_bndry);
bndry_pairs = [south_bndry_pairs, south_bndry_pairs+1;
    east_bndry_pairs, ...
    east_bndry_pairs+size(sp_seg,1)];

src_dst_pairs = [south_sp_src south_sp_dst; ...
    east_sp_src east_sp_dst];

[src_dst_pairs, sorted_ind] = sort(src_dst_pairs, 2);

sorted_ind = sub2ind(size(src_dst_pairs), ...
    repmat([1:size(sorted_ind,1)]',1,2), sorted_ind);

bndry_pairs = bndry_pairs(sorted_ind);

[edgelet_sp, ~, edgelet_ids] = unique(src_dst_pairs, 'rows');
end

function output = norm_chi2_dist(the_dist, select_pairs)
the_dist = bsxfun(@rdivide, double(the_dist), sum(the_dist,1));
K = chi_square_kernel(the_dist',the_dist',true);
idx = sub2ind(size(K),(select_pairs(:,1)),(select_pairs(:,2)));
output = K(idx);
end
