#include "omp.h"
#include "mex.h"

// img_masks = convert_masks(sp_seg, masks)
// Convert superpixel segments into image segments
// sp_seg is the superpixel map
// masks is the superpixel segments
//
// Changes (Ahmad Humayun):
// Fall 2014
//  Parallelized main loop using omp
//
// @authors:     Fuxin Li, Ahmad Humayun
// @contact:     ahumayun@cc.gatech.edu
// @affiliation: Georgia Institute of Technology
// @date:        Fall 2013 - Summer 2014
        
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
	// read the parameters
	// check input
	int idx_sp, idx;
	unsigned short *sp_seg;
	bool *masks;
	int nrows, ncols, nsegms, nsups;
	mwSize *dims = new mwSize[3];
	if( nrhs != 2 || mxGetClassID(prhs[0]) != mxUINT16_CLASS || mxGetClassID(prhs[1]) != mxLOGICAL_CLASS)
		mexErrMsgTxt("Usage: convert_masks(sp_seg, masks), sp_seg is the superpixel map (uint16) and masks are the superpixel segments.\n");
	
	nrows = mxGetM(prhs[0]);
	ncols = mxGetN(prhs[0]);
	nsegms = mxGetN(prhs[1]);
	nsups = mxGetM(prhs[1]);
	// convert the points to double
	dims[0] = nrows;
	dims[1] = ncols;
	dims[2] = nsegms;
	plhs[0] = mxCreateLogicalArray(3, dims);
	bool* segms = (bool *)mxGetData(plhs[0]);
	sp_seg = (unsigned short *) mxGetData(prhs[0]);
	masks = (bool *) mxGetData(prhs[1]);
#pragma omp parallel for schedule(dynamic,10) private(idx_sp,idx)
	for(int k=0; k < nsegms; ++k) {
		idx_sp = 0;
		bool* curr_segm_i = segms + k*nrows*ncols;
		for(int i=0; i < nrows*ncols; ++i) {
			short the_superpix = sp_seg[idx_sp];
			*curr_segm_i = masks[the_superpix - 1 + k * nsups];
			++idx_sp;
			++curr_segm_i;
		}
	}
}
